%{

#include "syslog-ng.h"
#include "utils.h"

#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "cfg-grammar.h"

struct keyword {
	char	*kw_name;
	int	kw_token;
};

static struct keyword keywords[] = {
	/* statements */
	{ "source", 		KW_SOURCE },
	{ "destination",	KW_DESTINATION },
	{ "filter",             KW_FILTER },
	{ "log",		KW_LOG },
	{ "options",		KW_OPTIONS },
       
	/* source or destination items */
	{ "file",		KW_FILE },
	{ "mark",		KW_MARK_FREQ },
        { "internal",           KW_INTERNAL },
	{ "unix-dgram",		KW_UNIX_DGRAM },
	{ "unix-stream",	KW_UNIX_STREAM },
        { "udp",                KW_UDP },
        { "tcp",                KW_TCP },

	/* filter items */
        { "or",                 KW_OR },
	{ "and",                KW_AND },
        { "not",                KW_NOT },
	{ "level",              KW_LEVEL },
	{ "facility",           KW_FACILITY },
	{ "program",		KW_PROGRAM }
};

int linenum;
int lex_filter_params = 0;

static int check_reserved_words(char *token);

%}

white	[ \r\t\n]
digit	[0-9]
alpha		[a-zA-Z]
alphanum	[a-zA-Z0-9]

%%

\n			   { linenum++; }
{white}+		   ;
\.\.                       { return DOTDOT; }
[0-9]+\.[0-9]+\.[0-9]+.[0-9]+ {
                            if (inet_aton(yytext, &yylval.ip)) 
                                     return IPADDR;
                             else
				     REJECT;
			   }
{digit}+		   { yylval.num = atoi(yytext); return NUMBER; }
[^ \.\(\)\{\}\\;\n\t,|]+   { return check_reserved_words(yytext); }
\(	      		   { return '('; }
\)			   { return ')'; }
\;			   { return ';'; }
\{			   { return '{'; }
\}			   { return '}'; }
\|			   { return '|'; }
\,			   { return ','; }

%%

int lex_init(FILE *file)
{
	yyrestart(file);
	linenum = 0;
	return 0;
}

int check_reserved_words(char *token)
{
	int i;
	
	for (i = 0; i < (sizeof(keywords) / sizeof(struct keyword)); i++) {
		if (strcmp(keywords[i].kw_name, token) == 0) {
			return keywords[i].kw_token;
		}
	}
	yylval.cptr = strdup(token);
	return IDENTIFIER;
}
